// DieRoll.cpp : Implementation of CDieRoll
#include "stdafx.h"
#include "DieRollControl.h"
#include "DieRoll.h"

/////////////////////////////////////////////////////////////////////////////
// CDieRoll

STDMETHODIMP CDieRoll::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IDieRoll,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

HRESULT CDieRoll::OnDraw(ATL_DRAWINFO& di)
{
    int width = (di.prcBounds->right - di.prcBounds->left + 1);
    int height = (di.prcBounds->bottom - di.prcBounds->top + 1); 

    BOOL drawn = FALSE;
    if (m_nReadyState == READYSTATE_COMPLETE)
    {
        if (BitmapDataLoaded)
        {
            hBitmap = ::CreateDIBitmap(di.hdcDraw, &bmih, CBM_INIT, lpvBits, 
                lpbmi, DIB_RGB_COLORS);

            if (hBitmap)
            {
                HDC hmemdc;
                hmemdc = ::CreateCompatibleDC(di.hdcDraw);
                ::SelectObject(hmemdc, hBitmap);
                DIBSECTION ds;
                ::GetObject(hBitmap,sizeof(DIBSECTION),(LPSTR)&ds);
                ::StretchBlt(di.hdcDraw,
                                di.prcBounds->left, // left
                                di.prcBounds->top,  // top
                                width, // target width
                                height, // target height
                                hmemdc,        // the image
                                0,            //offset into image -x
                                0,            //offset into image -y
                                ds.dsBm.bmWidth, // width
                                ds.dsBm.bmHeight, // height
                                SRCCOPY);    //copy it over

                drawn = TRUE;
                ::DeleteObject(hBitmap);
                hBitmap = NULL;
                ::DeleteDC(hmemdc);
            }
            else
            {
                GlobalUnlock(hmem1);
                GlobalFree(hmem1);
                GlobalUnlock(hmem2);
                GlobalFree(hmem2);
                BitmapDataLoaded = FALSE;
            }
        }
    }
    if (!drawn)
    {
        COLORREF back;
        ::OleTranslateColor(m_clrBackColor, NULL, &back);
        HBRUSH backbrush = ::CreateSolidBrush(back);
        ::FillRect(di.hdcDraw, (RECT *)di.prcBounds, backbrush);
        ::DeleteObject(backbrush);
    }
    COLORREF fore;
    ::OleTranslateColor(m_clrForeColor, NULL, &fore);
     if (!m_bDots)
     {
        _TCHAR val[20]; //character representation of the short value
        _itot(m_sNumber, val, 10);
            ::SetTextColor(di.hdcDraw, fore);
        ::ExtTextOut(di.hdcDraw, 0, 0, ETO_OPAQUE, 
            (RECT *)di.prcBounds, val, _tcslen(val), NULL );  
     }
     else
     {
         //dots are 4 units wide and high, one unit from the edge
         int Xunit = width/16;
         int Yunit = height/16;
         int Xleft = width%16;
         int Yleft = height%16;

         // adjust top left by amount left over
         int Top = di.prcBounds->top + Yleft/2;
         int Left = di.prcBounds->left + Xleft/2; 

         HBRUSH forebrush;
         forebrush = ::CreateSolidBrush(fore);

         HBRUSH savebrush = (HBRUSH)::SelectObject(di.hdcDraw, forebrush);

         switch(m_sNumber)
         {
         case 1:
             ::Ellipse(di.hdcDraw, Left+6*Xunit, Top+6*Yunit,
                            Left+10*Xunit, Top + 10*Yunit); //center
              break;
         case 2:
              ::Ellipse(di.hdcDraw, Left+Xunit, Top+Yunit,
                            Left+5*Xunit, Top + 5*Yunit);   //upper left
              ::Ellipse(di.hdcDraw, Left+11*Xunit, Top+11*Yunit,
                            Left+15*Xunit, Top + 15*Yunit); //lower right
              break;
         case 3:
              ::Ellipse(di.hdcDraw, Left+Xunit, Top+Yunit,
                            Left+5*Xunit, Top + 5*Yunit);   //upper left
              ::Ellipse(di.hdcDraw, Left+6*Xunit, Top+6*Yunit,
                            Left+10*Xunit, Top + 10*Yunit); //center
              ::Ellipse(di.hdcDraw, Left+11*Xunit, Top+11*Yunit,
                            Left+15*Xunit, Top + 15*Yunit); //lower right
              break;
         case 4:
              ::Ellipse(di.hdcDraw, Left+Xunit, Top+Yunit,
                            Left+5*Xunit, Top + 5*Yunit);   //upper left
              ::Ellipse(di.hdcDraw, Left+11*Xunit, Top+Yunit,
                            Left+15*Xunit, Top + 5*Yunit);  //upper right
              ::Ellipse(di.hdcDraw, Left+Xunit, Top+11*Yunit,
                            Left+5*Xunit, Top + 15*Yunit);  //lower left
              ::Ellipse(di.hdcDraw, Left+11*Xunit, Top+11*Yunit,
                            Left+15*Xunit, Top + 15*Yunit); //lower right
              break;
         case 5:
              ::Ellipse(di.hdcDraw, Left+Xunit, Top+Yunit,
                            Left+5*Xunit, Top + 5*Yunit);   //upper left
              ::Ellipse(di.hdcDraw, Left+11*Xunit, Top+Yunit,
                            Left+15*Xunit, Top + 5*Yunit);  //upper right
              ::Ellipse(di.hdcDraw, Left+6*Xunit, Top+6*Yunit,
                            Left+10*Xunit, Top + 10*Yunit); //center
              ::Ellipse(di.hdcDraw, Left+Xunit, Top+11*Yunit,
                            Left+5*Xunit, Top + 15*Yunit);  //lower left
              ::Ellipse(di.hdcDraw, Left+11*Xunit, Top+11*Yunit,
                            Left+15*Xunit, Top + 15*Yunit); //lower right
              break;
         case 6:
               ::Ellipse(di.hdcDraw, Left+Xunit, Top+Yunit,
                    Left+5*Xunit, Top + 5*Yunit);   //upper left
               ::Ellipse(di.hdcDraw, Left+11*Xunit, Top+Yunit,
                    Left+15*Xunit, Top + 5*Yunit);  //upper right
               ::Ellipse(di.hdcDraw, Left+Xunit, Top+6*Yunit,
                    Left+5*Xunit, Top + 10*Yunit);  //center left
               ::Ellipse(di.hdcDraw, Left+11*Xunit, Top+6*Yunit,
                      Left+15*Xunit, Top + 10*Yunit); //center right
               ::Ellipse(di.hdcDraw, Left+Xunit, Top+11*Yunit,
                    Left+5*Xunit, Top + 15*Yunit);  //lower left
               ::Ellipse(di.hdcDraw, Left+11*Xunit, Top+11*Yunit,
                    Left+15*Xunit, Top + 15*Yunit); //lower right
               break;
         }

         ::SelectObject(di.hdcDraw, savebrush);
         ::DeleteObject(forebrush);
     }
    return S_OK;
}


STDMETHODIMP CDieRoll::get_Number(short * pVal)
{
	*pVal = m_sNumber;
	return S_OK;
}

STDMETHODIMP CDieRoll::get_Dots(BOOL * pVal)
{
	*pVal = m_bDots;
	return S_OK;
}

STDMETHODIMP CDieRoll::put_Dots(BOOL newVal)
{
	if (FireOnRequestEdit(dispidDots) == S_FALSE)
	{
		return S_FALSE;
	}
	m_bDots = newVal;
	SetDirty(TRUE);
	FireOnChanged(dispidDots);
	FireViewChange();

	return S_OK;
}

short CDieRoll::Roll()
{
     double number = rand();
     number /= RAND_MAX + 1;
     number *= 6;
     return (short)number + 1;
}

STDMETHODIMP CDieRoll::get_Image(BSTR * pVal)
{
    *pVal = m_bstrImage.Copy();
    return S_OK;
}

STDMETHODIMP CDieRoll::put_Image(BSTR newVal)
{
    USES_CONVERSION;

    if (FireOnRequestEdit(dispidImage) == S_FALSE)
    {
        return S_FALSE;
    }

    // if there was an old bitmap or data, delete them
	if (BitmapDataLoaded)
    {
        GlobalUnlock(hmem1);
        GlobalFree(hmem1);
        GlobalUnlock(hmem2);
        GlobalFree(hmem2);
        BitmapDataLoaded = FALSE;
    }

    if (m_Data != NULL)
    {
        delete m_Data;
    }

    m_Data = NULL;
    m_DataLength = 0;

	m_bstrImage = newVal;
    LPSTR string = W2A(m_bstrImage);

	if (string != NULL && strlen(string) > 0)
	{
		// not a null string so try to load it
	    BOOL relativeURL = FALSE;
	    if (strchr(string, ':') == NULL)
		{
			relativeURL = TRUE;
		}

	    m_nReadyState = READYSTATE_LOADING;
		Fire_ReadyStateChange();

	    HRESULT ret = CBindStatusCallback<CDieRoll>::Download(this, 
		    OnData, m_bstrImage, m_spClientSite, relativeURL);
	}
	else
	{
		// was a null string so don't try to load it
	    m_nReadyState = READYSTATE_INTERACTIVE;
		Fire_ReadyStateChange();
	}

    SetDirty(TRUE);
    FireOnChanged(dispidImage);
    return S_OK;
}

void CDieRoll::OnData(CBindStatusCallback<CDieRoll>* pbsc, 
                      BYTE * pBytes, DWORD dwSize)
{
    char *newData = new char[m_DataLength + dwSize];

    memcpy(newData, m_Data, m_DataLength);
    memcpy(newData+m_DataLength, pBytes, dwSize);
    m_DataLength += dwSize;

    delete m_Data;
    m_Data = newData;

    if (ReadBitmap())
    {
        m_nReadyState = READYSTATE_COMPLETE;
		Fire_ReadyStateChange();
        FireViewChange();
    }
}

BOOL CDieRoll::ReadBitmap()
{
	BITMAPFILEHEADER bmfh;

	unsigned long cur_offset = 0;

	/* Retrieve the BITMAPFILEHEADER structure. */ 
 	if (cur_offset + sizeof(BITMAPFILEHEADER) > m_DataLength)
	{
		// return FALSE to indicate an error occured
		return FALSE;
	}
 	memcpy(&bmfh, m_Data+cur_offset, sizeof(BITMAPFILEHEADER));
	cur_offset += sizeof(BITMAPFILEHEADER);
 
	/* Retrieve the BITMAPINFOHEADER structure. */ 
 	if (cur_offset + sizeof(BITMAPINFOHEADER) > m_DataLength)
	{
		// return FALSE to indicate an error occured
		return FALSE;
	}
 	memcpy(&bmih, m_Data+cur_offset, sizeof(BITMAPINFOHEADER));
	cur_offset += sizeof(BITMAPINFOHEADER);

	
	/*
	 *   The colour array size is calculated by taking the size of
	 *   the file and subtracting off the size of the image and the
	 *   size of the headers
	 *       still do not like this as it can be wrong with a 
	 *       damaged file. Also it is hard to say if it will
	 *       always work. But it does work for more cases then
	 *       the old code. 
	 */

    if (bmih.biSizeImage + sizeof(BITMAPFILEHEADER) + bmih.biSize
				> m_DataLength)
	{
		// colour_size would be negative so return
		return FALSE;
	}

	unsigned long colour_size = m_DataLength - bmih.biSizeImage -
		    sizeof(BITMAPFILEHEADER) - bmih.biSize;

	/* Allocate memory for the BITMAPINFO structure. */ 
	hmem1 = GlobalAlloc(GHND, sizeof(BITMAPINFOHEADER) + colour_size); 
 
	/* Lock it */
	lpbmi = (BITMAPINFO *)GlobalLock(hmem1); 
 
	/* 
	 * Load BITMAPINFOHEADER into the BITMAPINFO 
	 * structure. 
	 */ 
 
	lpbmi->bmiHeader.biSize = bmih.biSize; 
	lpbmi->bmiHeader.biWidth = bmih.biWidth; 
	lpbmi->bmiHeader.biHeight = bmih.biHeight; 
	lpbmi->bmiHeader.biPlanes = bmih.biPlanes; 
	lpbmi->bmiHeader.biBitCount = bmih.biBitCount; 
	lpbmi->bmiHeader.biCompression = bmih.biCompression; 
	lpbmi->bmiHeader.biSizeImage = bmih.biSizeImage; 
	lpbmi->bmiHeader.biXPelsPerMeter = bmih.biXPelsPerMeter; 
	lpbmi->bmiHeader.biYPelsPerMeter = bmih.biYPelsPerMeter; 
	lpbmi->bmiHeader.biClrUsed = bmih.biClrUsed; 
	lpbmi->bmiHeader.biClrImportant = bmih.biClrImportant; 
 
	if (cur_offset + colour_size > m_DataLength)
	{
		// return FALSE to indicate an error occured
		GlobalUnlock(hmem1); 
		GlobalFree(hmem1);
		return FALSE;
	}
	memcpy(lpbmi->bmiColors, m_Data+cur_offset, colour_size);
	cur_offset += colour_size;

	/* 
	 * Allocate memory for the required number of 
	 * bytes. 
	 */ 
 
	hmem2 = GlobalAlloc(GHND, (bmfh.bfSize - bmfh.bfOffBits)); 
 
	lpvBits = (char *)GlobalLock(hmem2); 
 
	/* Retrieve the bitmap data. */ 
 
 	if (cur_offset + (bmfh.bfSize - bmfh.bfOffBits) > m_DataLength)
	{
		// return FALSE to indicate an error occured
		GlobalUnlock(hmem1); 
		GlobalFree(hmem1);
		GlobalUnlock(hmem2); 
		GlobalFree(hmem2);
		return FALSE;
	}
 	memcpy(lpvBits, m_Data+cur_offset, (bmfh.bfSize - bmfh.bfOffBits));
	cur_offset += (bmfh.bfSize - bmfh.bfOffBits);

	BitmapDataLoaded = TRUE;

	return TRUE;
}

LRESULT CDieRoll::OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled)
{
	m_sNumber = Roll();
	FireOnChanged(dispidNumber);
   	Fire_Click();
	FireViewChange();

    return 0;
}

STDMETHODIMP CDieRoll::DoRoll()
{
	m_sNumber = Roll();
	FireOnChanged(dispidNumber);
	FireViewChange();
	return S_OK;
} 

HRESULT CDieRoll::FireViewChange()
{
	if (m_bInPlaceActive)
	{
		// Active
		if (m_hWndCD != NULL)
			return ::InvalidateRect(m_hWndCD, NULL, FALSE); // Window based
		if (m_spInPlaceSite != NULL)
			return m_spInPlaceSite->InvalidateRect(NULL, FALSE); // Windowless
	}
	// Inactive
	SendOnViewChange(DVASPECT_CONTENT);
	return S_OK;

}
