// DieRoll.h : Declaration of the CDieRoll

#ifndef __DIEROLL_H_
#define __DIEROLL_H_

#include "resource.h"       // main symbols
#include "time.h"
#include "CPDieRollControl.h"

/////////////////////////////////////////////////////////////////////////////
// CDieRoll
class ATL_NO_VTABLE CDieRoll : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CDieRoll, &CLSID_DieRoll>,
	public CComControl<CDieRoll>,
	public CStockPropImpl<CDieRoll, IDieRoll, &IID_IDieRoll, &LIBID_DIEROLLCONTROLLib>,
	public IProvideClassInfo2Impl<&CLSID_DieRoll, NULL, &LIBID_DIEROLLCONTROLLib>,
	public IPersistStreamInitImpl<CDieRoll>,
	public IPersistStorageImpl<CDieRoll>,
	public IQuickActivateImpl<CDieRoll>,
	public IOleControlImpl<CDieRoll>,
	public IOleObjectImpl<CDieRoll>,
	public IOleInPlaceActiveObjectImpl<CDieRoll>,
	public IViewObjectExImpl<CDieRoll>,
	public IOleInPlaceObjectWindowlessImpl<CDieRoll>,
	public IDataObjectImpl<CDieRoll>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CDieRoll>,
	public IPersistPropertyBagImpl<CDieRoll>,
	public CProxy_DieRollEvents<CDieRoll>,
	public IObjectSafetyImpl<CDieRoll>,
	public ISpecifyPropertyPagesImpl<CDieRoll>
{
public:
	CDieRoll()
	{
        srand( (unsigned)time( NULL ) );
        m_nReadyState = READYSTATE_INTERACTIVE;
        m_clrBackColor = 0x80000000 | COLOR_WINDOW;
        m_clrForeColor = 0x80000000 | COLOR_WINDOWTEXT;
        m_sNumber = Roll();
        m_bDots = TRUE;
        m_Data = NULL;
        m_DataLength = 0;
        BitmapDataLoaded = FALSE;
	}

    ~CDieRoll()
    {
        if (BitmapDataLoaded)
        {
            GlobalUnlock(hmem1);
            GlobalFree(hmem1);
            GlobalUnlock(hmem2);
            GlobalFree(hmem2);
            BitmapDataLoaded = FALSE;
        }

        if (m_Data != NULL)
        {
            delete m_Data;
        }
    }

    STDMETHOD(GetInterfaceSafetyOptions)(REFIID riid, 
          DWORD *pdwSupportedOptions, DWORD *pdwEnabledOptions)
    {
        ATLTRACE(_T("IObjectSafetyImpl::GetInterfaceSafetyOptions\n"));
        if (pdwSupportedOptions == NULL || pdwEnabledOptions == NULL)
            return E_POINTER;
        *pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER || INTERFACESAFE_FOR_UNTRUSTED_DATA;
        *pdwEnabledOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER || INTERFACESAFE_FOR_UNTRUSTED_DATA;
        return S_OK;
    }

    STDMETHOD(SetInterfaceSafetyOptions)(REFIID riid, DWORD dwOptionSetMask, DWORD dwEnabledOptions)
    {
        ATLTRACE(_T("IObjectSafetyImpl::SetInterfaceSafetyOptions\n"));
        return S_OK;
    }

DECLARE_REGISTRY_RESOURCEID(IDR_DIEROLL)

BEGIN_COM_MAP(CDieRoll)
	COM_INTERFACE_ENTRY(IDieRoll)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY_IMPL(IViewObjectEx)
	COM_INTERFACE_ENTRY_IMPL_IID(IID_IViewObject2, IViewObjectEx)
	COM_INTERFACE_ENTRY_IMPL_IID(IID_IViewObject, IViewObjectEx)
	COM_INTERFACE_ENTRY_IMPL(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IMPL_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IMPL_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IMPL(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY_IMPL(IOleControl)
	COM_INTERFACE_ENTRY_IMPL(IOleObject)
	COM_INTERFACE_ENTRY_IMPL(IQuickActivate)
	COM_INTERFACE_ENTRY_IMPL(IPersistStorage)
	COM_INTERFACE_ENTRY_IMPL(IPersistStreamInit)
	COM_INTERFACE_ENTRY_IMPL(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY_IMPL(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IMPL(IObjectSafety)
	COM_INTERFACE_ENTRY_IMPL(IPersistPropertyBag)
END_COM_MAP()

BEGIN_PROPERTY_MAP(CDieRoll)
    PROP_ENTRY( "Dots", dispidDots, CLSID_DieRollPPG)
    PROP_ENTRY( "Image", dispidImage, CLSID_DieRollPPG)
    PROP_ENTRY( "Fore Color", DISPID_FORECOLOR, CLSID_StockColorPage )
    PROP_ENTRY( "Back Color", DISPID_BACKCOLOR, CLSID_StockColorPage )
END_PROPERTY_MAP()


BEGIN_CONNECTION_POINT_MAP(CDieRoll)
    CONNECTION_POINT_ENTRY(DIID__DieRollEvents)
END_CONNECTION_POINT_MAP()


BEGIN_MSG_MAP(CDieRoll)
	MESSAGE_HANDLER(WM_PAINT, OnPaint)
	MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
	MESSAGE_HANDLER(WM_KILLFOCUS, OnKillFocus)
	MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)
END_MSG_MAP()


// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IViewObjectEx
	STDMETHOD(GetViewStatus)(DWORD* pdwStatus)
	{
		ATLTRACE(_T("IViewObjectExImpl::GetViewStatus\n"));
		*pdwStatus = VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE;
		return S_OK;
	}

// IDieRoll
public:
	HRESULT FireViewChange();
	STDMETHOD(DoRoll)();
	STDMETHOD(get_Image)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_Image)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_Dots)(/*[out, retval]*/ BOOL *pVal);
	STDMETHOD(put_Dots)(/*[in]*/ BOOL newVal);
	STDMETHOD(get_Number)(/*[out, retval]*/ short *pVal);
    void OnData(CBindStatusCallback<CDieRoll>* pbsc, BYTE * pBytes, DWORD dwSize);

	HRESULT OnDraw(ATL_DRAWINFO& di);
	LRESULT OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL & bHandled);


	OLE_COLOR m_clrBackColor;
	OLE_COLOR m_clrForeColor;
	long m_nReadyState;
	short m_sNumber;
	BOOL m_bDots;
	CComBSTR m_bstrImage;

	HBITMAP hBitmap;
	BITMAPINFOHEADER bmih;
	char *lpvBits;
	BITMAPINFO *lpbmi;
	HGLOBAL hmem1;
	HGLOBAL hmem2;
	BOOL BitmapDataLoaded;
	char *m_Data;
	unsigned long m_DataLength;


protected:
	short Roll();
    BOOL ReadBitmap();
};

#endif //__DIEROLL_H_
