// ShowStringView.cpp : implementation of the CShowStringView class
//

#include "stdafx.h"
#include "ShowString.h"

#include "ShowStringDoc.h"
#include "CntrItem.h"
#include "ShowStringView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CShowStringView

IMPLEMENT_DYNCREATE(CShowStringView, CView)

BEGIN_MESSAGE_MAP(CShowStringView, CView)
	//{{AFX_MSG_MAP(CShowStringView)
	ON_WM_DESTROY()
	ON_WM_SETFOCUS()
	ON_WM_SIZE()
	ON_COMMAND(ID_OLE_INSERT_NEW, OnInsertObject)
	ON_COMMAND(ID_CANCEL_EDIT_CNTR, OnCancelEditCntr)
	ON_WM_LBUTTONDOWN()
	ON_WM_SETCURSOR()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_CREATE()
	ON_COMMAND(ID_EDIT_CLEAR, OnEditClear)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEAR, OnUpdateEditClear)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CShowStringView construction/destruction

CLIPFORMAT CShowStringView::m_cfObjectDescriptorFormat =
     (CLIPFORMAT) ::RegisterClipboardFormat("Object Descriptor");

CShowStringView::CShowStringView()
{
     m_pSelection = NULL;
     m_FocusRectangleDrawn = FALSE;
}

CShowStringView::~CShowStringView()
{
}

CShowStringCntrItem* CShowStringView::HitTest(CPoint point)
{
     CShowStringDoc* pDoc = GetDocument();
     CShowStringCntrItem* pHitItem = NULL;

     POSITION pos = pDoc->GetStartPosition();
     while (pos)
     {
          CShowStringCntrItem* pCurrentItem = 
               (CShowStringCntrItem*) pDoc->GetNextClientItem(pos);
          if ( pCurrentItem->m_rect.PtInRect(point) )
          {
               pHitItem = pCurrentItem;
          }
     }

     return pHitItem;
}

void CShowStringView::SetupTracker(CShowStringCntrItem* item, 
     CRectTracker* track)
{
     track->m_rect = item->m_rect;

     if (item == m_pSelection)
     {
          track->m_nStyle |= CRectTracker::resizeInside;
     }

     if (item->GetType() == OT_LINK)
     {
          track->m_nStyle |= CRectTracker::dottedLine;
     }
     else
     {
          track->m_nStyle |= CRectTracker::solidLine;
     }
     if (item->GetItemState() == COleClientItem::openState ||
          item->GetItemState() == COleClientItem::activeUIState)
     {
          track->m_nStyle |= CRectTracker::hatchInside;
     }
}

BOOL CShowStringView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CShowStringView drawing

void CShowStringView::OnDraw(CDC* pDC)
{
	CShowStringDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
    CRect rect;
    GetClientRect(&rect);
    COLORREF oldcolor;
    switch (pDoc->GetColor())
    {
    case 0:
        oldcolor = pDC->SetTextColor(RGB(0,0,0)); //black
        break;
    case 1:
        oldcolor = pDC->SetTextColor(RGB(0xFF,0,0)); //red
        break;
    case 2:
        oldcolor = pDC->SetTextColor(RGB(0,0xFF,0)); //green
        break;
    }

    int DTflags = 0;
    if (pDoc->GetHorizcenter())
    {
        DTflags |= DT_CENTER;
    }
    if (pDoc->GetVertcenter())
    {
        DTflags |= (DT_VCENTER|DT_SINGLELINE);
    }


    pDC->DrawText(pDoc->GetString(), &rect, DTflags);

    pDC->SetTextColor(oldcolor);

     POSITION pos = pDoc->GetStartPosition();
     while (pos)
     {
          CShowStringCntrItem* pCurrentItem = 
               (CShowStringCntrItem*) pDoc->GetNextClientItem(pos);
          pCurrentItem->Draw(pDC, pCurrentItem->m_rect);

          if (pCurrentItem == m_pSelection )
          {
               CRectTracker trackrect;
               SetupTracker(pCurrentItem,&trackrect);
               trackrect.Draw(pDC);
          }
     }
}

void CShowStringView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	// TODO: remove this code when final selection model code is written
	m_pSelection = NULL;    // initialize selection

}

/////////////////////////////////////////////////////////////////////////////
// CShowStringView printing

BOOL CShowStringView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CShowStringView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CShowStringView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CShowStringView::OnDestroy()
{
	// Deactivate the item on destruction; this is important
	// when a splitter view is being used.
   CView::OnDestroy();
   COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
   if (pActiveItem != NULL && pActiveItem->GetActiveView() == this)
   {
      pActiveItem->Deactivate();
      ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
   }
}


/////////////////////////////////////////////////////////////////////////////
// OLE Client support and commands

BOOL CShowStringView::IsSelected(const CObject* pDocItem) const
{
	// The implementation below is adequate if your selection consists of
	//  only CShowStringCntrItem objects.  To handle different selection
	//  mechanisms, the implementation here should be replaced.

	// TODO: implement this function that tests for a selected OLE client item

	return pDocItem == m_pSelection;
}

void CShowStringView::OnInsertObject()
{
	// Invoke the standard Insert Object dialog box to obtain information
	//  for new CShowStringCntrItem object.
	COleInsertDialog dlg;
	if (dlg.DoModal() != IDOK)
		return;

	BeginWaitCursor();

	CShowStringCntrItem* pItem = NULL;
	TRY
	{
		// Create new item connected to this document.
		CShowStringDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		pItem = new CShowStringCntrItem(pDoc);
		ASSERT_VALID(pItem);

		// Initialize the item from the dialog data.
		if (!dlg.CreateItem(pItem))
			AfxThrowMemoryException();  // any exception will do
		ASSERT_VALID(pItem);

		// If item created from class list (not from file) then launch
		//  the server to edit the item.
		if (dlg.GetSelectionType() == COleInsertDialog::createNewItem)
			pItem->DoVerb(OLEIVERB_SHOW, this);

		ASSERT_VALID(pItem);

		// As an arbitrary user interface design, this sets the selection
		//  to the last item inserted.

		// TODO: reimplement selection as appropriate for your application

		m_pSelection = pItem;   // set selection to last inserted item
		pDoc->UpdateAllViews(NULL);
	}
	CATCH(CException, e)
	{
		if (pItem != NULL)
		{
			ASSERT_VALID(pItem);
			pItem->Delete();
		}
		AfxMessageBox(IDP_FAILED_TO_CREATE);
	}
	END_CATCH

	EndWaitCursor();
}

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the container (not the server) causes the deactivation.
void CShowStringView::OnCancelEditCntr()
{
	// Close any in-place active item on this view.
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL)
	{
		pActiveItem->Close();
	}
	ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
}

// Special handling of OnSetFocus and OnSize are required for a container
//  when an object is being edited in-place.
void CShowStringView::OnSetFocus(CWnd* pOldWnd)
{
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL &&
		pActiveItem->GetItemState() == COleClientItem::activeUIState)
	{
		// need to set focus to this item if it is in the same view
		CWnd* pWnd = pActiveItem->GetInPlaceWindow();
		if (pWnd != NULL)
		{
			pWnd->SetFocus();   // don't call the base class
			return;
		}
	}

	CView::OnSetFocus(pOldWnd);
}

void CShowStringView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL)
		pActiveItem->SetItemRects();
}

/////////////////////////////////////////////////////////////////////////////
// CShowStringView diagnostics

#ifdef _DEBUG
void CShowStringView::AssertValid() const
{
	CView::AssertValid();
}

void CShowStringView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CShowStringDoc* CShowStringView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CShowStringDoc)));
	return (CShowStringDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CShowStringView message handlers

void CShowStringView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CShowStringCntrItem* pHitItem = HitTest(point);
	SetSelection(pHitItem);
	if (pHitItem == NULL)
		return;
	
	CRectTracker track;
	SetupTracker(pHitItem, &track);
	UpdateWindow();

	if (track.HitTest(point) == CRectTracker::hitMiddle)
	{
		CRect rect =  pHitItem->m_rect;
		CClientDC dc(this);
		OnPrepareDC(&dc);
		dc.LPtoDP(&rect); // convert logical rect to device rect
		rect.NormalizeRect();
		CPoint newpoint = point - rect.TopLeft();

		DROPEFFECT dropEffect = pHitItem->DoDragDrop(rect, newpoint);
		if (dropEffect == DROPEFFECT_MOVE)
		{
			Invalidate();
			if (pHitItem == m_pSelection)
			{
				m_pSelection = NULL;
			}
			pHitItem->Delete();
		}	
	}
	else
	{
		if (track.Track(this,point))
		{
			Invalidate();
			pHitItem->m_rect = track.m_rect;
			GetDocument()->SetModifiedFlag();
		}
	}
}

void CShowStringView::SetSelection(CShowStringCntrItem* item)
{
     // if an item is being edited in place, close it
     if ( item == NULL || item != m_pSelection)
     {
          COleClientItem* pActive = 
               GetDocument()->GetInPlaceActiveItem(this);
          if (pActive != NULL && pActive != item)
          {
               pActive->Close();
          }
     }
     Invalidate();
     m_pSelection = item;
}

BOOL CShowStringView::OnSetCursor(CWnd* pWnd, UINT nHitTest, 
     UINT message) 
{
     if (pWnd == this && m_pSelection != NULL)
     {
          CRectTracker track;
          SetupTracker(m_pSelection, &track);
          if (track.SetCursor(this, nHitTest))
          {
               return TRUE;
          }
     }
     
	return CView::OnSetCursor(pWnd, nHitTest, message);
}

void CShowStringView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
     OnLButtonDown(nFlags, point);     

     if( m_pSelection)
     {
          if (GetKeyState(VK_CONTROL) < 0)
          {
               m_pSelection->DoVerb(OLEIVERB_OPEN, this);
          }
          else
          {
               m_pSelection->DoVerb(OLEIVERB_PRIMARY, this);
          }
     }

     CView::OnLButtonDblClk(nFlags, point);
}

int CShowStringView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
     if (CView::OnCreate(lpCreateStruct) == -1)
          return -1;
     
     if (m_droptarget.Register(this))
     {
          return 0;
     }
     else
     {
          return -1;
     }
}

DROPEFFECT CShowStringView::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 
{
	ASSERT(!m_FocusRectangleDrawn);
	
	// check that the data object can be dropped in this view
	m_OKtodrop = FALSE;
	if (!COleClientItem::CanCreateFromData(pDataObject))
		return DROPEFFECT_NONE;

	m_OKtodrop = TRUE;

	// set dragsize and dragoffset with call to GetGlobalData
	HGLOBAL hObjectDescriptor = pDataObject->GetGlobalData(
		m_cfObjectDescriptorFormat);
	if (hObjectDescriptor)
	{
		LPOBJECTDESCRIPTOR pObjectDescriptor = 
			(LPOBJECTDESCRIPTOR) GlobalLock(hObjectDescriptor);
		ASSERT(pObjectDescriptor);
		m_dragsize.cx = (int) pObjectDescriptor->sizel.cx;
		m_dragsize.cy = (int) pObjectDescriptor->sizel.cy;
		m_dragoffset.cx = (int) pObjectDescriptor->pointl.x;
		m_dragoffset.cy = (int) pObjectDescriptor->pointl.y;
		GlobalUnlock(hObjectDescriptor);
		GlobalFree(hObjectDescriptor);
	}
	else
	{
		m_dragsize = CSize(0,0);
		m_dragoffset = CSize(0,0);
	}
	// convert sizes with a scratch dc
	CClientDC dc(NULL);
	dc.HIMETRICtoDP(&m_dragsize);
	dc.HIMETRICtoDP(&m_dragoffset);
	// hand off to OnDragOver
	return OnDragOver(pDataObject, dwKeyState, point);
}

DROPEFFECT CShowStringView::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 
{
	// return if dropping is already rejected
	if (!m_OKtodrop)
	{
		return DROPEFFECT_NONE;
	}
	
	// determine drop effect according to keys depressed
	DROPEFFECT dropeffect = DROPEFFECT_NONE;

	if ((dwKeyState & (MK_CONTROL|MK_SHIFT) )
		== (MK_CONTROL|MK_SHIFT))
	{
		// Ctrl+Shift force a link
		dropeffect = DROPEFFECT_LINK;
	}

	else if ((dwKeyState & MK_CONTROL)	== MK_CONTROL)
	{
		// Ctrl forces a copy
		dropeffect = DROPEFFECT_COPY;
	}
	else if ((dwKeyState & MK_ALT) == MK_ALT)
	{
		// Alt forces a move
		dropeffect = DROPEFFECT_MOVE;
	}
	else
	{
		// default is to move
		dropeffect = DROPEFFECT_MOVE;
	}

	// adjust focus rectangle

	point -= m_dragoffset;
	if (point == m_dragpoint)
	{
		return dropeffect;
	}
	
	CClientDC dc(this);

	if (m_FocusRectangleDrawn)
	{
		dc.DrawFocusRect(CRect(m_dragpoint, m_dragsize));
		m_FocusRectangleDrawn = FALSE;
	}

	if (dropeffect != DROPEFFECT_NONE)
	{
		dc.DrawFocusRect(CRect(point, m_dragsize));
		m_dragpoint = point;
		m_FocusRectangleDrawn = TRUE;
	}

	return dropeffect;

}

void CShowStringView::OnDragLeave() 
{
	CClientDC dc(this);
	if (m_FocusRectangleDrawn)
	{
		dc.DrawFocusRect(CRect(m_dragpoint, m_dragsize));
		m_FocusRectangleDrawn = FALSE;
	}
}

BOOL CShowStringView::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point) 
{
	ASSERT_VALID(this);

	// remove focus rectangle
	CClientDC dc(this);
	if (m_FocusRectangleDrawn)
	{
		dc.DrawFocusRect(CRect(m_dragpoint, m_dragsize));
		m_FocusRectangleDrawn = FALSE;
	}

	// paste in the data object
	CShowStringDoc* pDoc = GetDocument();
	CShowStringCntrItem* pNewItem = new CShowStringCntrItem(pDoc);
	ASSERT_VALID(pNewItem);
	if (dropEffect & DROPEFFECT_LINK)
	{
		pNewItem->CreateLinkFromData(pDataObject);
	}
	else
	{
		pNewItem->CreateFromData(pDataObject);
	}
	ASSERT_VALID(pNewItem);

	// adjust the item dimensions, and make it the current selection
	CSize size;
	pNewItem->GetExtent(&size, pNewItem->GetDrawAspect());
	dc.HIMETRICtoDP(&size);
	point -= m_dragoffset;
	pNewItem->m_rect = CRect(point,size);
	m_pSelection = pNewItem;

	// update views and set modified flag
	pDoc->SetModifiedFlag();
	pDoc->UpdateAllViews(NULL);
	return TRUE;

}

void CShowStringView::OnEditClear() 
{
     if (m_pSelection)
     {
          m_pSelection->Delete();
          m_pSelection = NULL;
          GetDocument()->SetModifiedFlag();
          GetDocument()->UpdateAllViews(NULL);
     }
}

void CShowStringView::OnUpdateEditClear(CCmdUI* pCmdUI) 
{
     pCmdUI->Enable(m_pSelection != NULL);
}
